package com.nisum.offertransactionservice.service;

import com.nisum.offertransactionservice.genericexception.GlobalApiGenericException;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.PropertySource;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Service;
import org.springframework.web.reactive.function.client.ExchangeFilterFunction;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Mono;

import javax.annotation.PostConstruct;

import java.util.Objects;

import static com.nisum.offertransactionservice.util.ExceptionUtil.handleError;

@Service
@PropertySource("classpath:application.properties")
@Slf4j
public class EndOfTransactionService {

    private WebClient webClient;

    @Value("${store.consumer.url}")
    private String storeConsumerUrl;

    @Value("${store.consumer.baseUrl}")
    private String baseUrl;

    @PostConstruct
    public void init() {
        webClient = WebClient.builder().baseUrl(baseUrl).
                defaultHeader(HttpHeaders.CONTENT_TYPE, MediaType.APPLICATION_JSON_VALUE).
                filter(logRequest()).
                filter(logResponse()).
                build();
    }


    private ExchangeFilterFunction logRequest() {
        return ExchangeFilterFunction.ofRequestProcessor(clientRequest -> {
            log.info("Store Consumer Request {}", clientRequest);
            return Mono.just(clientRequest);
        });
    }

    private ExchangeFilterFunction logResponse() {
        return ExchangeFilterFunction.ofResponseProcessor(clientResponse -> {
            log.info("Store Consumer Request {}", clientResponse);
            return Mono.just(clientResponse);
        });
    }

    public String endOfTransaction(String uuid) {
        if (Objects.isNull(uuid) || uuid.isEmpty()) {
            log.error("Offer transaction uuid is empty or null");
            throw new GlobalApiGenericException("Unable to get the Offer transaction uuid ", new Exception("exception e"), "400", false);
        }
        log.info("Store Consumer WebClient call Start");
        Mono<String> endOfTransactionRes = webClient.post().
                uri(storeConsumerUrl).
                accept(MediaType.APPLICATION_JSON).
                contentType(MediaType.APPLICATION_JSON).
                body(Mono.just(uuid), String.class).
                retrieve().
                onStatus(HttpStatus::is4xxClientError, clientResponse -> handleError(clientResponse)).
                onStatus(HttpStatus::is5xxServerError, clientResponse -> handleError(clientResponse)).
                bodyToMono(String.class);
        log.debug("Store consumer WebClient call End");
        return endOfTransactionRes.toString();
    }


}
